@@ -240,8 +240,7 @@ MatroskaTrack* MatroskaFile::TrackTable::lookup(unsigned trackNumber) {
 
 unsigned MatroskaFile::TrackTable::numTracks() const { return fTable->numEntries(); }
 
-MatroskaFile::TrackTable::Iterator::Iterator(MatroskaFile::TrackTable& ourTable)
-  : fOurTable(ourTable) {
+MatroskaFile::TrackTable::Iterator::Iterator(MatroskaFile::TrackTable& ourTable) {
   fIter = HashTable::Iterator::create(*(ourTable.fTable));
 }
 
@@ -264,8 +263,7 @@ MatroskaTrack::MatroskaTrack()
     name(NULL), language(NULL), codecID(NULL),
     samplingFrequency(0), numChannels(2), mimeType(""),
     codecPrivateSize(0), codecPrivate(NULL), headerStrippedBytesSize(0), headerStrippedBytes(NULL),
-    subframeSizeSize(0), durationImbalance(0) {
-  prevPresentationTime.tv_sec = 0; prevPresentationTime.tv_usec = 0;
+    subframeSizeSize(0) {
 }
 
 MatroskaTrack::~MatroskaTrack() {
