@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A parser for a Matroska file.
 // Implementation
 
@@ -100,8 +100,7 @@ void MatroskaFileParser::continueParsing() {
     }
   }
 
-  // We successfully parsed the file's 'Track' headers (or else there was no input source, because the specified file name
-  // didn't exist).  Call our 'done' function now:
+  // We successfully parsed the file's 'Track' headers.  Call our 'done' function now:
   if (fOnEndFunc != NULL) (*fOnEndFunc)(fOnEndClientData);
 }
 
@@ -932,24 +931,25 @@ Boolean MatroskaFileParser::deliverFrameWithinBlock() {
 
     if (track->defaultDuration == 0) {
       // Adjust the frame duration to keep the sum of frame durations aligned with presentation times.
-      if (track->prevPresentationTime.tv_sec != 0) { // not the first time for this track
-	track->durationImbalance
-	  += (presentationTime.tv_sec - track->prevPresentationTime.tv_sec)*1000000
-	  + (presentationTime.tv_usec - track->prevPresentationTime.tv_usec);
+      if (demuxedTrack->prevPresentationTime().tv_sec != 0) { // not the first time for this track
+	demuxedTrack->durationImbalance()
+	  += (presentationTime.tv_sec - demuxedTrack->prevPresentationTime().tv_sec)*1000000
+	  + (presentationTime.tv_usec - demuxedTrack->prevPresentationTime().tv_usec);
       }
       int adjustment = 0;
-      if (track->durationImbalance > 0) {
+      if (demuxedTrack->durationImbalance() > 0) {
 	// The duration needs to be increased.
 	int const adjustmentThreshold = 100000; // don't increase the duration by more than this amount (in case there's a mistake)
-	adjustment = track->durationImbalance > adjustmentThreshold ? adjustmentThreshold : track->durationImbalance;
-      } else if (track->durationImbalance < 0) {
+	adjustment = demuxedTrack->durationImbalance() > adjustmentThreshold
+	  ? adjustmentThreshold : demuxedTrack->durationImbalance();
+      } else if (demuxedTrack->durationImbalance() < 0) {
 	// The duration needs to be decreased.
-	adjustment
-	  = (unsigned)(-track->durationImbalance) < durationInMicroseconds ? track->durationImbalance : -(int)durationInMicroseconds;
+	adjustment = (unsigned)(-demuxedTrack->durationImbalance()) < durationInMicroseconds
+	  ? demuxedTrack->durationImbalance() : -(int)durationInMicroseconds;
       }
       durationInMicroseconds += adjustment;
-      track->durationImbalance -= durationInMicroseconds; // for next time
-      track->prevPresentationTime = presentationTime; // for next time
+      demuxedTrack->durationImbalance() -= durationInMicroseconds; // for next time
+      demuxedTrack->prevPresentationTime() = presentationTime; // for next time
     }
 
     demuxedTrack->presentationTime() = presentationTime;
@@ -1133,7 +1133,8 @@ Boolean MatroskaFileParser::parseEBMLVal_float(EBMLDataSize& size, float& result
   unsigned resultAsUnsigned;
   if (!parseEBMLVal_unsigned(size, resultAsUnsigned)) return False;
 
-  result = *(float*)&resultAsUnsigned;
+  if (sizeof result != sizeof resultAsUnsigned) return False;
+  memcpy(&result, &resultAsUnsigned, sizeof result);
   return True;
 }
 
