@@ -20,6 +20,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Implementation
 
 #include "liveMedia.hh"
+#include "RTSPCommon.hh"
 #include "GroupsockHelper.hh" // for "our_random()"
 
 #ifndef MILLION
@@ -48,7 +49,6 @@ private:
   void subsessionByeHandler();
 
   int verbosityLevel() const { return ((ProxyServerMediaSession*)fParentSession)->fVerbosityLevel; }
-  void reset();
 
 private:
   friend class ProxyRTSPClient;
@@ -87,6 +87,14 @@ ProxyServerMediaSession::ProxyServerMediaSession(UsageEnvironment& env, RTSPServ
 }
 
 ProxyServerMediaSession::~ProxyServerMediaSession() {
+  if (fVerbosityLevel > 0) {
+    envir() << *this << "::~ProxyServerMediaSession()\n";
+  }
+
+  // Begin by sending a "TEARDOWN" command (without checking for a response):
+  if (fProxyRTSPClient != NULL) fProxyRTSPClient->sendTeardownCommand(*fClientMediaSession, NULL, fProxyRTSPClient->auth());
+
+  // Then delete our state:
   Medium::close(fClientMediaSession);
   Medium::close(fProxyRTSPClient);
   delete fPresentationTimeSessionNormalizer;
@@ -125,15 +133,15 @@ void ProxyServerMediaSession::continueAfterDESCRIBE(char const* sdpDescription)
 }
 
 void ProxyServerMediaSession::resetDESCRIBEState() {
-  // Delete the client "MediaSession" object that we had set up after receiving the response to the previous "DESCRIBE":
-  Medium::close(fClientMediaSession); fClientMediaSession = NULL;
-
-  // Also delete all of our "ProxyServerMediaSubsession"s; they'll get set up again once we get a response to the new "DESCRIBE".
+  // Delete all of our "ProxyServerMediaSubsession"s; they'll get set up again once we get a response to the new "DESCRIBE".
   if (fOurRTSPServer != NULL) {
     // First, close any RTSP client connections that may have already been set up:
     fOurRTSPServer->closeAllClientSessionsForServerMediaSession(this);
   }
   deleteAllSubsessions();
+
+  // Finally, delete the client "MediaSession" object that we had set up after receiving the response to the previous "DESCRIBE":
+  Medium::close(fClientMediaSession); fClientMediaSession = NULL;
 }
 
 ///////// RTSP 'response handlers' //////////
@@ -160,10 +168,22 @@ static void continueAfterSETUP(RTSPClient* rtspClient, int resultCode, char* res
 }
 
 static void continueAfterOPTIONS(RTSPClient* rtspClient, int resultCode, char* resultString) {
-  ((ProxyRTSPClient*)rtspClient)->continueAfterOPTIONS(resultCode);
+  Boolean serverSupportsGetParameter = False;
+  if (resultCode == 0) {
+    // Note whether the server told us that it supports the "GET_PARAMETER" command:
+    serverSupportsGetParameter = RTSPOptionIsSupported("GET_PARAMETER", resultString);
+  }
+  ((ProxyRTSPClient*)rtspClient)->continueAfterLivenessCommand(resultCode, serverSupportsGetParameter);
   delete[] resultString;
 }
 
+#ifdef SEND_GET_PARAMETER_IF_SUPPORTED
+static void continueAfterGET_PARAMETER(RTSPClient* rtspClient, int resultCode, char* resultString) {
+  ((ProxyRTSPClient*)rtspClient)->continueAfterLivenessCommand(resultCode, True);
+  delete[] resultString;
+}
+#endif
+
 
 ////////// "ProxyRTSPClient" implementation /////////
 
@@ -176,7 +196,8 @@ ProxyRTSPClient::ProxyRTSPClient(ProxyServerMediaSession& ourServerMediaSession,
   : RTSPClient(ourServerMediaSession.envir(), rtspURL, verbosityLevel, "ProxyRTSPClient",
 	       tunnelOverHTTPPortNum == (portNumBits)(~0) ? 0 : tunnelOverHTTPPortNum),
     fOurServerMediaSession(ourServerMediaSession), fOurURL(strDup(rtspURL)), fStreamRTPOverTCP(tunnelOverHTTPPortNum != 0),
-    fSetupQueueHead(NULL), fSetupQueueTail(NULL), fNumSetupsDone(0), fNextDESCRIBEDelay(1), fLastCommandWasPLAY(False),
+    fSetupQueueHead(NULL), fSetupQueueTail(NULL), fNumSetupsDone(0), fNextDESCRIBEDelay(1),
+    fServerSupportsGetParameter(False), fLastCommandWasPLAY(False),
     fLivenessCommandTask(NULL), fDESCRIBECommandTask(NULL), fSubsessionTimerTask(NULL) { 
   if (username != NULL && password != NULL) {
     fOurAuthenticator = new Authenticator(username, password);
@@ -211,9 +232,9 @@ void ProxyRTSPClient::continueAfterDESCRIBE(char const* sdpDescription) {
 
     // Unlike most RTSP streams, there might be a long delay between this "DESCRIBE" command (to the downstream server) and the
     // subsequent "SETUP"/"PLAY" - which doesn't occur until the first time that a client requests the stream.
-    // To prevent the proxied connection (between us and the downstream server) from timing out, we send periodic
-    // "OPTIONS" commands.  (The usual RTCP liveness mechanism wouldn't work here, because RTCP packets don't get sent
-    // until after the "PLAY" command.)
+    // To prevent the proxied connection (between us and the downstream server) from timing out, we send periodic 'liveness'
+    // ("OPTIONS" or "GET_PARAMETER") commands.  (The usual RTCP liveness mechanism wouldn't work here, because RTCP packets
+    // don't get sent until after the "PLAY" command.)
     scheduleLivenessCommand();
   } else {
     // The "DESCRIBE" command failed, most likely because the server or the stream is not yet running.
@@ -222,26 +243,34 @@ void ProxyRTSPClient::continueAfterDESCRIBE(char const* sdpDescription) {
   }
 }
 
-void ProxyRTSPClient::continueAfterOPTIONS(int resultCode) {
-  if (resultCode < 0) {
-    // The "OPTIONS" command failed without getting a response from the server (otherwise "resultCode" would have been >= 0).
-    // From this, we infer that the server (which had previously been running) has now failed - perhaps temporarily.
+void ProxyRTSPClient::continueAfterLivenessCommand(int resultCode, Boolean serverSupportsGetParameter) {
+  if (resultCode != 0) {
+    // The periodic 'liveness' command failed, suggesting that the back-end stream is no longer alive.
     // We handle this by resetting our connection state with this server.  Any current clients will be closed, but
     // subsequent clients will cause new RTSP "SETUP"s and "PLAY"s to get done, restarting the stream.
-    if (fVerbosityLevel > 0) {
-      envir() << *this << ": lost connection to server ('errno': " << -resultCode << ").  Resetting...\n";
-    }
-    reset();
+    // Then continue by sending more "DESCRIBE" commands, to try to restore the stream.
 
+    fServerSupportsGetParameter = False; // until we learn otherwise, in response to a future "OPTIONS" command
+
+    if (resultCode < 0) {
+      // The 'liveness' command failed without getting a response from the server (otherwise "resultCode" would have been > 0).
+      // This suggests that the RTSP connection itself has failed.  Print this error code, in case it's useful for debugging:
+      if (fVerbosityLevel > 0) {
+	envir() << *this << ": lost connection to server ('errno': " << -resultCode << ").  Resetting...\n";
+      }
+    }
+
+    reset();
     fOurServerMediaSession.resetDESCRIBEState();
 
-    // In case the back-end server comes alive again, try to restore the back-end connection by sending more "DESCRIBE" commands.
     setBaseURL(fOurURL); // because we'll be sending an initial "DESCRIBE" all over again
     sendDESCRIBE(this);
     return;
   }
 
-  // Schedule the next RTSP "OPTIONS" command (to show client liveness):
+  fServerSupportsGetParameter = serverSupportsGetParameter;
+
+  // Schedule the next 'liveness' command (i.e., to tell the back-end server that we're still alive):
   scheduleLivenessCommand();
 }
 
@@ -290,14 +319,35 @@ void ProxyRTSPClient::continueAfterSETUP() {
 }
 
 void ProxyRTSPClient::scheduleLivenessCommand() {
-  // Delay a random time before sending "GET_PARAMETER":
-  unsigned secondsToDelay = 30 + (our_random()&0x1F); // [30..61] seconds
-  fLivenessCommandTask = envir().taskScheduler().scheduleDelayedTask(secondsToDelay*MILLION, sendLivenessCommand, this);
+  // Delay a random time before sending another 'liveness' command.
+  unsigned delayMax = sessionTimeoutParameter(); // if the server specified a maximum time between 'liveness' probes, then use that
+  if (delayMax == 0) {
+    delayMax = 60;
+  }
+
+  // Choose a random time from [delayMax/2,delayMax) seconds:
+  unsigned const dM5 = delayMax*500000;
+  unsigned uSecondsToDelay = dM5 + (dM5*our_random())%dM5;
+  fLivenessCommandTask = envir().taskScheduler().scheduleDelayedTask(uSecondsToDelay, sendLivenessCommand, this);
 }
 
 void ProxyRTSPClient::sendLivenessCommand(void* clientData) {
   ProxyRTSPClient* rtspClient = (ProxyRTSPClient*)clientData;
-  rtspClient->sendOptionsCommand(::continueAfterOPTIONS, rtspClient->auth());
+
+  // Note.  By default, we do not send "GET_PARAMETER" as our 'liveness notification' command, even if the server previously
+  // indicated (in its response to our earlier "OPTIONS" command) that it supported "GET_PARAMETER".  This is because
+  // "GET_PARAMETER" crashes some camera servers (even though they claimed to support "GET_PARAMETER").
+#ifdef SEND_GET_PARAMETER_IF_SUPPORTED
+  MediaSession* sess = rtspClient->fOurServerMediaSession.fClientMediaSession;
+
+  if (rtspClient->fServerSupportsGetParameter && rtspClient->fNumSetupsDone > 0 && sess != NULL) {
+    rtspClient->sendGetParameterCommand(*sess, ::continueAfterGET_PARAMETER, "", rtspClient->auth());
+  } else {
+#endif
+    rtspClient->sendOptionsCommand(::continueAfterOPTIONS, rtspClient->auth());
+#ifdef SEND_GET_PARAMETER_IF_SUPPORTED
+  }
+#endif
 }
 
 void ProxyRTSPClient::scheduleDESCRIBECommand() {
@@ -340,13 +390,16 @@ ProxyServerMediaSubsession::ProxyServerMediaSubsession(MediaSubsession& mediaSub
     fClientMediaSubsession(mediaSubsession), fNext(NULL), fHaveSetupStream(False) {
 }
 
-ProxyServerMediaSubsession::~ProxyServerMediaSubsession() {
-}
-
 UsageEnvironment& operator<<(UsageEnvironment& env, const ProxyServerMediaSubsession& psmss) { // used for debugging
   return env << "ProxyServerMediaSubsession[\"" << psmss.codecName() << "\"]";
 }
 
+ProxyServerMediaSubsession::~ProxyServerMediaSubsession() {
+  if (verbosityLevel() > 0) {
+    envir() << *this << "::~ProxyServerMediaSubsession()\n";
+  }
+}
+
 FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientSessionId, unsigned& estBitrate) {
   ProxyServerMediaSession* const sms = (ProxyServerMediaSession*)fParentSession;
 
@@ -357,6 +410,7 @@ FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientS
   // If we haven't yet created a data source from our 'media subsession' object, initiate() it to do so:
   if (fClientMediaSubsession.readSource() == NULL) {
     fClientMediaSubsession.receiveRawMP3ADUs(); // hack for MPA-ROBUST streams
+    fClientMediaSubsession.receiveRawJPEGFrames(); // hack for proxying JPEG/RTP streams. (Don't do this if we're transcoding.)
     fClientMediaSubsession.initiate();
     if (verbosityLevel() > 0) {
       envir() << "\tInitiated: " << *this << "\n";
@@ -365,12 +419,13 @@ FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientS
     if (fClientMediaSubsession.readSource() != NULL) {
       // Add to the front of all data sources a filter that will 'normalize' their frames' presentation times,
       // before the frames get re-transmitted by our server:
+      char const* const codecName = fClientMediaSubsession.codecName();
       FramedFilter* normalizerFilter = sms->fPresentationTimeSessionNormalizer
-	->createNewPresentationTimeSubsessionNormalizer(fClientMediaSubsession.readSource(), fClientMediaSubsession.rtpSource());
+	->createNewPresentationTimeSubsessionNormalizer(fClientMediaSubsession.readSource(), fClientMediaSubsession.rtpSource(),
+							codecName);
       fClientMediaSubsession.addFilter(normalizerFilter);
 
       // Some data sources require a 'framer' object to be added, before they can be fed into a "RTPSink".  Adjust for this now:
-      char const* const codecName = fClientMediaSubsession.codecName();
       if (strcmp(codecName, "H264") == 0) {
 	fClientMediaSubsession.addFilter(H264VideoStreamDiscreteFramer::createNew(envir(), fClientMediaSubsession.readSource()));
       } else if (strcmp(codecName, "MP4V-ES") == 0) {
@@ -460,10 +515,12 @@ RTPSink* ProxyServerMediaSubsession
   if (strcmp(codecName, "AC3") == 0 || strcmp(codecName, "EAC3") == 0) {
     newSink = AC3AudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					 fClientMediaSubsession.rtpTimestampFrequency()); 
+#if 0 // This code does not work; do *not* enable it:
   } else if (strcmp(codecName, "AMR") == 0 || strcmp(codecName, "AMR-WB") == 0) {
     Boolean isWideband = strcmp(codecName, "AMR-WB") == 0;
     newSink = AMRAudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					 isWideband, fClientMediaSubsession.numChannels());
+#endif
   } else if (strcmp(codecName, "DV") == 0) {
     newSink = DVVideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
   } else if (strcmp(codecName, "GSM") == 0) {
@@ -474,6 +531,9 @@ RTPSink* ProxyServerMediaSubsession
   } else if (strcmp(codecName, "H264") == 0) {
     newSink = H264VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					  fClientMediaSubsession.fmtp_spropparametersets());
+  } else if (strcmp(codecName, "JPEG") == 0) {
+    newSink = SimpleRTPSink::createNew(envir(), rtpGroupsock, 26, 90000, "video", "JPEG",
+				       1/*numChannels*/, False/*allowMultipleFramesPerPacket*/, False/*doNormalMBitRule*/);
   } else if (strcmp(codecName, "MP4A-LATM") == 0) {
     newSink = MPEG4LATMAudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
 					       fClientMediaSubsession.rtpTimestampFrequency(),
@@ -502,6 +562,14 @@ RTPSink* ProxyServerMediaSubsession
 					    fClientMediaSubsession.fmtp_config()); 
   } else if (strcmp(codecName, "VP8") == 0) {
     newSink = VP8VideoRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic);
+  } else if (strcmp(codecName, "AMR") == 0 || strcmp(codecName, "AMR-WB") == 0) {
+    // Proxying of these codecs is currently *not* supported, because the data received by the "RTPSource" object is not in a
+    // form that can be fed directly into a corresponding "RTPSink" object.
+    if (verbosityLevel() > 0) {
+      envir() << "\treturns NULL (because we currently don't support the proxying of \""
+	      << fClientMediaSubsession.mediumName() << "/" << codecName << "\" streams)\n";
+    }
+    return NULL;
   } else if (strcmp(codecName, "QCELP") == 0 ||
 	     strcmp(codecName, "H261") == 0 ||
 	     strcmp(codecName, "H263-1998") == 0 || strcmp(codecName, "H263-2000") == 0 ||
@@ -512,7 +580,7 @@ RTPSink* ProxyServerMediaSubsession
     }
     return NULL;
   } else {
-    // This codec is assumed to have a simple RTP paylaod format that can be implemented just with a "SimpleRTPSink":
+    // This codec is assumed to have a simple RTP payload format that can be implemented just with a "SimpleRTPSink":
     Boolean allowMultipleFramesPerPacket = True; // by default
     Boolean doNormalMBitRule = True; // by default
     // Some codecs change the above default parameters:
@@ -556,11 +624,9 @@ void ProxyServerMediaSubsession::subsessionByeHandler() {
 
   // This "BYE" signals that our input source has (effectively) closed, so handle this accordingly:
   FramedSource::handleClosure(fClientMediaSubsession.readSource());
-}
 
-void ProxyServerMediaSubsession::reset() {
-  fNext = NULL;
-  fHaveSetupStream = False;
+  // Then, close our input source for real:
+  fClientMediaSubsession.deInitiate();
 }
 
 
@@ -580,8 +646,10 @@ PresentationTimeSessionNormalizer::~PresentationTimeSessionNormalizer() {
 }
 
 PresentationTimeSubsessionNormalizer*
-PresentationTimeSessionNormalizer::createNewPresentationTimeSubsessionNormalizer(FramedSource* inputSource, RTPSource* rtpSource) {
-  fSubsessionNormalizers = new PresentationTimeSubsessionNormalizer(*this, inputSource, rtpSource, fSubsessionNormalizers);
+PresentationTimeSessionNormalizer::createNewPresentationTimeSubsessionNormalizer(FramedSource* inputSource, RTPSource* rtpSource,
+										 char const* codecName) {
+  fSubsessionNormalizers
+    = new PresentationTimeSubsessionNormalizer(*this, inputSource, rtpSource, codecName, fSubsessionNormalizers);
   return fSubsessionNormalizers;
 }
 
@@ -638,9 +706,9 @@ void PresentationTimeSessionNormalizer
 
 PresentationTimeSubsessionNormalizer
 ::PresentationTimeSubsessionNormalizer(PresentationTimeSessionNormalizer& parent, FramedSource* inputSource, RTPSource* rtpSource,
-				       PresentationTimeSubsessionNormalizer* next)
+				       char const* codecName, PresentationTimeSubsessionNormalizer* next)
   : FramedFilter(parent.envir(), inputSource),
-    fParent(parent), fRTPSource(rtpSource), fRTPSink(NULL), fNext(next) {
+    fParent(parent), fRTPSource(rtpSource), fRTPSink(NULL), fCodecName(codecName), fNext(next) {
 }
 
 PresentationTimeSubsessionNormalizer::~PresentationTimeSubsessionNormalizer() {
@@ -666,6 +734,10 @@ void PresentationTimeSubsessionNormalizer::afterGettingFrame(unsigned frameSize,
 
   fParent.normalizePresentationTime(this, fPresentationTime, presentationTime);
 
+  // Hack for JPEG/RTP proxying.  Because we're proxying JPEG by just copying the raw JPEG/RTP payloads, without interpreting them,
+  // we need to also 'copy' the RTP 'M' (marker) bit from the "RTPSource" to the "RTPSink":
+  if (fRTPSource->curPacketMarkerBit() && strcmp(fCodecName, "JPEG") == 0) ((SimpleRTPSink*)fRTPSink)->setMBitOnNextPacket();
+
   // Complete delivery:
   FramedSource::afterGetting(this);
 }
