@@ -354,10 +354,10 @@ void RTSPServer::RTSPClientConnection::handleCmd_OPTIONS() {
 
 void RTSPServer::RTSPClientConnection
 ::handleCmd_GET_PARAMETER(char const* /*fullRequestStr*/) {
-  // By default, we implement "GET_PARAMETER" (on the entire server) just as a 'no op', and send back an empty response.
+  // By default, we implement "GET_PARAMETER" (on the entire server) just as a 'no op', and send back a dummy response.
   // (If you want to handle this type of "GET_PARAMETER" differently, you can do so by defining a subclass of "RTSPServer"
   // and "RTSPServer::RTSPClientConnection", and then reimplement this virtual function in your subclass.)
-  setRTSPResponse("200 OK");
+  setRTSPResponse("200 OK", LIVEMEDIA_LIBRARY_VERSION_STRING);
 }
 
 void RTSPServer::RTSPClientConnection
@@ -728,15 +728,15 @@ void RTSPServer::RTSPClientConnection::handleRequestBytes(int newBytesRead) {
     char cseq[RTSP_PARAM_STRING_MAX];
     char sessionIdStr[RTSP_PARAM_STRING_MAX];
     unsigned contentLength = 0;
-    *fLastCRLF = '\0'; // temporarily, for parsing
-    Boolean parseSucceeded = parseRTSPRequestString((char*)fRequestBuffer, fRequestBytesAlreadySeen,
+    fLastCRLF[2] = '\0'; // temporarily, for parsing
+    Boolean parseSucceeded = parseRTSPRequestString((char*)fRequestBuffer, fLastCRLF+2 - fRequestBuffer,
 						    cmdName, sizeof cmdName,
 						    urlPreSuffix, sizeof urlPreSuffix,
 						    urlSuffix, sizeof urlSuffix,
 						    cseq, sizeof cseq,
 						    sessionIdStr, sizeof sessionIdStr,
 						    contentLength);
-    *fLastCRLF = '\r';
+    fLastCRLF[2] = '\r'; // restore its value
     if (parseSucceeded) {
 #ifdef DEBUG
       fprintf(stderr, "parseRTSPRequestString() succeeded, returning cmdName \"%s\", urlPreSuffix \"%s\", urlSuffix \"%s\", CSeq \"%s\", Content-Length %u, with %d bytes following the message.\n", cmdName, urlPreSuffix, urlSuffix, cseq, contentLength, ptr + newBytesRead - (tmpPtr + 2));
@@ -1034,6 +1034,44 @@ void RTSPServer::RTSPClientConnection
 	   sessionId);
 }
 
+void RTSPServer::RTSPClientConnection
+::setRTSPResponse(char const* responseStr, char const* contentStr) {
+  if (contentStr == NULL) contentStr = "";
+  unsigned const contentLen = strlen(contentStr);
+
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+	   "RTSP/1.0 %s\r\n"
+	   "CSeq: %s\r\n"
+	   "%s"
+	   "Content-Length: %d\r\n\r\n"
+	   "%s",
+	   responseStr,
+	   fCurrentCSeq,
+	   dateHeader(),
+	   contentLen,
+	   contentStr);
+}
+
+void RTSPServer::RTSPClientConnection
+::setRTSPResponse(char const* responseStr, u_int32_t sessionId, char const* contentStr) {
+  if (contentStr == NULL) contentStr = "";
+  unsigned const contentLen = strlen(contentStr);
+
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+	   "RTSP/1.0 %s\r\n"
+	   "CSeq: %s\r\n"
+	   "%s"
+	   "Session: %08X\r\n"
+	   "Content-Length: %d\r\n\r\n"
+	   "%s",
+	   responseStr,
+	   fCurrentCSeq,
+	   dateHeader(),
+	   sessionId,
+	   contentLen,
+	   contentStr);
+}
+
 void RTSPServer::RTSPClientConnection
 ::changeClientInputSocket(int newSocketNum, unsigned char const* extraData, unsigned extraDataSize) {
   envir().taskScheduler().disableBackgroundHandling(fClientInputSocket);
@@ -1638,6 +1676,11 @@ void RTSPServer::RTSPClientSession
 						    rangeStart, streamDuration, numBytes);
 	  }
 	}
+      } else {
+	// No "Range:" header was specified in the "PLAY", so we do a 'null' seek (i.e., we don't seek at all):
+	if (fStreamStates[i].subsession != NULL) {
+	  fStreamStates[i].subsession->nullSeekStream(fOurSessionId, fStreamStates[i].streamToken);
+	}
       }
     }
   }
@@ -1646,7 +1689,21 @@ void RTSPServer::RTSPClientSession
   // (Note that we do this after seeking, in case the seeking operation changed the range start time.)
   char* rangeHeader;
   if (!sawRangeHeader) {
-    buf[0] = '\0'; // Because we didn't see a Range: header, don't send one back
+    // There wasn't a "Range:" header in the request, so, in our response, begin the range with the current NPT (normal play time):
+    float curNPT = 0.0;
+    for (i = 0; i < fNumStreamStates; ++i) {
+      if (subsession == NULL /* means: aggregated operation */
+	  || subsession == fStreamStates[i].subsession) {
+	if (fStreamStates[i].subsession == NULL) continue;
+	float npt = fStreamStates[i].subsession->getCurrentNPT(fStreamStates[i].streamToken);
+	if (npt > curNPT) curNPT = npt;
+	// Note: If this is an aggregate "PLAY" on a multi-subsession stream, then it's conceivable that the NPTs of each subsession
+	// may differ (if there has been a previous seek on just one subsession).  In this (unusual) case, we just return the
+	// largest NPT; I hope that turns out OK...
+      }
+    }
+
+    sprintf(buf, "Range: npt=%.3f-\r\n", curNPT);
   } else if (absStart != NULL) {
     // We're seeking by 'absolute' time:
     if (absEnd == NULL) {
@@ -1743,10 +1800,10 @@ void RTSPServer::RTSPClientSession
 void RTSPServer::RTSPClientSession
 ::handleCmd_GET_PARAMETER(RTSPServer::RTSPClientConnection* ourClientConnection,
 			  ServerMediaSubsession* /*subsession*/, char const* /*fullRequestStr*/) {
-  // By default, we implement "GET_PARAMETER" just as a 'keep alive', and send back an empty response.
+  // By default, we implement "GET_PARAMETER" just as a 'keep alive', and send back a dummy response.
   // (If you want to handle "GET_PARAMETER" properly, you can do so by defining a subclass of "RTSPServer"
   // and "RTSPServer::RTSPClientSession", and then reimplement this virtual function in your subclass.)
-  ourClientConnection->setRTSPResponse("200 OK", fOurSessionId);
+  ourClientConnection->setRTSPResponse("200 OK", fOurSessionId, LIVEMEDIA_LIBRARY_VERSION_STRING);
 }
 
 void RTSPServer::RTSPClientSession
