@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // A generic SIP client
 // Implementation
 
@@ -49,7 +49,8 @@ SIPClient::SIPClient(UsageEnvironment& env,
     fCSeq(0), fURL(NULL), fURLSize(0),
     fToTagStr(NULL), fToTagStrSize(0),
     fUserName(NULL), fUserNameSize(0),
-    fInviteSDPDescription(NULL), fInviteCmd(NULL), fInviteCmdSize(0){
+    fInviteSDPDescription(NULL), fInviteSDPDescriptionReturned(NULL),
+    fInviteCmd(NULL), fInviteCmdSize(0) {
   if (mimeSubtype == NULL) mimeSubtype = "";
   fMIMESubtype = strDup(mimeSubtype);
   fMIMESubtypeSize = strlen(fMIMESubtype);
@@ -60,13 +61,13 @@ SIPClient::SIPClient(UsageEnvironment& env,
 
   struct in_addr ourAddress;
   ourAddress.s_addr = ourIPAddress(env); // hack
-  fOurAddressStr = strDup(our_inet_ntoa(ourAddress));
+  fOurAddressStr = strDup(AddressString(ourAddress).val());
   fOurAddressStrSize = strlen(fOurAddressStr);
 
   fOurSocket = new Groupsock(env, ourAddress, 0, 255);
   if (fOurSocket == NULL) {
     env << "ERROR: Failed to create socket for addr "
-	<< our_inet_ntoa(ourAddress) << ": "
+	<< fOurAddressStr << ": "
 	<< env.getResultMsg() << "\n";
   }
 
@@ -84,7 +85,7 @@ SIPClient::SIPClient(UsageEnvironment& env,
     fOurSocket = new Groupsock(env, ourAddress, fOurPortNum, 255);
     if (fOurSocket == NULL) {
       env << "ERROR: Failed to create socket for addr "
-	  << our_inet_ntoa(ourAddress) << ", port "
+	  << fOurAddressStr << ", port "
 	  << fOurPortNum << ": "
 	  << env.getResultMsg() << "\n";
     }
@@ -181,8 +182,8 @@ char* SIPClient::invite(char const* url, Authenticator* authenticator) {
   delete[] (char*)fURL; fURL = strDup(url);
   fURLSize = strlen(fURL);
 
-  fCallId = our_random();
-  fFromTag = our_random();
+  fCallId = our_random32();
+  fFromTag = our_random32();
 
   return invite1(authenticator);
 }
@@ -243,6 +244,7 @@ char* SIPClient::invite1(Authenticator* authenticator) {
       "INVITE %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s\r\n"
       "Contact: sip:%s@%s:%u\r\n"
       "Call-ID: %u@%s\r\n"
@@ -432,6 +434,7 @@ void SIPClient::doInviteStateTerminated(unsigned responseCode) {
   if (responseCode < 200 || responseCode > 299) {
     // We failed, so return NULL;
     delete[] fInviteSDPDescription; fInviteSDPDescription = NULL;
+    delete[] fInviteSDPDescriptionReturned; fInviteSDPDescriptionReturned = NULL;
   }
 
   // Unblock the event loop:
@@ -528,7 +531,7 @@ unsigned SIPClient::getResponseCode() {
       delete[] toTagStr;
 
       if (sscanf(lineStart, "Content-Length: %d", &contentLength) == 1
-          || sscanf(lineStart, "Content-Length: %d", &contentLength) == 1) {
+          || sscanf(lineStart, "Content-length: %d", &contentLength) == 1) {
         if (contentLength < 0) {
           envir().setResultMsg("Bad \"Content-Length:\" header: \"",
                                lineStart, "\"");
@@ -595,6 +598,7 @@ unsigned SIPClient::getResponseCode() {
       }
 
       bodyStart[contentLength] = '\0'; // trims any extra data
+      delete[] fInviteSDPDescriptionReturned; fInviteSDPDescriptionReturned = strDup(bodyStart);
     }
   } while (0);
 
@@ -636,6 +640,7 @@ Boolean SIPClient::sendACK() {
       "ACK %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s;tag=%s\r\n"
       "Call-ID: %u@%s\r\n"
       "CSeq: %d ACK\r\n"
@@ -677,9 +682,10 @@ Boolean SIPClient::sendBYE() {
       "BYE %s SIP/2.0\r\n"
       "From: %s <sip:%s@%s>;tag=%u\r\n"
       "Via: SIP/2.0/UDP %s:%u\r\n"
+      "Max-Forwards: 70\r\n"
       "To: %s;tag=%s\r\n"
       "Call-ID: %u@%s\r\n"
-      "CSeq: %d ACK\r\n"
+      "CSeq: %d BYE\r\n"
       "Content-Length: 0\r\n\r\n";
     unsigned cmdSize = strlen(cmdFmt)
       + fURLSize
