@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2012 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2013 Live Networks, Inc.  All rights reserved.
 // RTP sink for Vorbis audio
 // Implementation
 
@@ -126,7 +126,7 @@ VorbisAudioRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
 }
 
 #define ADVANCE(n) do { p += (n); rem -= (n); } while (0)
-#define GET_ENCODED_VAL(n) do { u_int8_t byte; n = 0; do { if (rem == 0) break; byte = *p; n = n*128 + byte&0x7F; ADVANCE(1); } while (byte&0x80); } while (0); if (rem == 0) break
+#define GET_ENCODED_VAL(n) do { u_int8_t byte; n = 0; do { if (rem == 0) break; byte = *p; n = (n*128) + (byte&0x7F); ADVANCE(1); } while (byte&0x80); } while (0); if (rem == 0) break
 
 VorbisAudioRTPSink*
 VorbisAudioRTPSink::createNew(UsageEnvironment& env, Groupsock* RTPgs,
