@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2012, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2013, Live Networks, Inc.  All rights reserved
 // A program that converts a H.264 (Elementary Stream) video file into a Transport Stream file.
 // main program
 
@@ -47,9 +47,8 @@ int main(int argc, char** argv) {
   MPEG2TransportStreamFromESSource* tsFrames = MPEG2TransportStreamFromESSource::createNew(*env);
   tsFrames->addNewVideoSource(framer, 5/*mpegVersion: H.264*/);
   
-  // Open the output file as a 'file sink'.  Give it a large buffer size, because the input H.264 NAL units may be large:
-  unsigned const fileSinkBufferSize = 100000;
-  MediaSink* outputSink = FileSink::createNew(*env, outputFileName, fileSinkBufferSize);
+  // Open the output file as a 'file sink':
+  MediaSink* outputSink = FileSink::createNew(*env, outputFileName);
   if (outputSink == NULL) {
     *env << "Unable to open file \"" << outputFileName << "\" as a file sink\n";
     exit(1);
